
var Filters = Filters || {};

// default sampling method is point sampling
// after implementing bilinear sampling, you may wish to change this to 'bilinear'
Filters.sampling = 'point';

Filters.samplePixel = function ( image, x, y ) {
    if ( this.sampling == 'bilinear') {
        // ----------- STUDENT CODE BEGIN ------------
        // ----------- STUDENT CODE END ------------
        Gui.alertOnce ('bilinear sampling is not implemented yet');

    } else if ( this.sampling == 'gaussian' ) {
        // ----------- STUDENT CODE BEGIN ------------
        // ----------- STUDENT CODE END ------------
        Gui.alertOnce ('gaussian sampling is not implemented yet');
        
    } else { // point sampling
        
        var i = Math.max( 0, Math.min(Math.round(x), image.height- 1) );
        var j = Math.max( 0, Math.min(Math.round(y), image.width - 1) );
        return image.getPixel(i, j);
    } 
}

Filters.brightnessFilter = function( image, ratio ) {
    var alpha, dirLuminance;
    if (ratio < 0.0) {
        alpha = 1 + ratio;
        dirLuminance = 0;   // blend with black
    } else {
        alpha = 1 - ratio;
        dirLuminance = 255; // blend with white
    }
    for (var i = 0; i < image.height; i++) {
        for (var j = 0; j < image.width; j++) {
            var pixel = image.getPixel(i, j);
            
            pixel.r = alpha * pixel.r + (1-alpha) * dirLuminance;
            pixel.g = alpha * pixel.g + (1-alpha) * dirLuminance;
            pixel.b = alpha * pixel.b + (1-alpha) * dirLuminance;

            image.setPixel(i, j, pixel)
        }
    }
    return image;
};

Filters.contrastFilter = function( image, ratio ) {
    // ----------- STUDENT CODE BEGIN ------------
    // ----------- STUDENT CODE END ------------
    Gui.alertOnce ('contrastFilter is not implemented yet');
    return image;

};

Filters.gammaFilter = function( image, logOfGamma ) {
    var gamma = Math.exp(logOfGamma);
    // ----------- STUDENT CODE BEGIN ------------
    // ----------- STUDENT CODE END ------------
    Gui.alertOnce ('gammaFilter is not implemented yet');
    return image;
    
};

Filters.vignetteFilter = function( image, value ) {
    var innerR, outterR;
    if (value.length > 1) {
        innerR  = 0.5 - 0.5 * value[0];
        outterR = 0.5 + 0.5 * value[1];
    } else {
        innerR  = 0.5 - 0.5 * value;
        outterR = 1;        
    }
    
    // ----------- STUDENT CODE BEGIN ------------
    // ----------- STUDENT CODE END ------------
    Gui.alertOnce ('vignetteFilter is not implemented yet');
    return image;
};

Filters.histeqFilter = function( image ) {
    // ----------- STUDENT CODE BEGIN ------------
    // ----------- STUDENT CODE END ------------
    Gui.alertOnce ('vignetteFilter is not implemented yet');
    return image;
};

Filters.grayFilter = function( image ) { 
    for (var i = 0; i < image.height; i++) {
        for (var j = 0; j < image.width; j++) {
            var pixel = image.getPixel(i, j);
            var luminance = 0.2126 * pixel.r + 0.7152 * pixel.g + 0.0722 * pixel.b;
            pixel.r = luminance;
            pixel.g = luminance;
            pixel.b = luminance;

            image.setPixel(i, j, pixel);
        }
    }

    return image;
};

Filters.saturationFilter = function( image, ratio ) {
    // ----------- STUDENT CODE BEGIN ------------
    // ----------- STUDENT CODE END ------------
    Gui.alertOnce ('saturationFilter is not implemented yet');
    return image;
}; 

Filters.whiteBalanceFilter = function( image, hex ) {
    // ----------- STUDENT CODE BEGIN ------------
    // ----------- STUDENT CODE END ------------
    Gui.alertOnce ('whiteBalanceFilter is not implemented yet');
    return image;
};

Filters.histMatchFilter = function( image, refImg, value ) {
    // ----------- STUDENT CODE BEGIN ------------
    // ----------- STUDENT CODE END ------------
    Gui.alertOnce ('histMatchFilter is not implemented yet');
    return image;
};

Filters.gaussianFilter = function( image, sigma ) {
    // note: this function needs to work in a new copy of the image
    //       to avoid overwriting original pixels values needed later 
    // create a new image with the same size as the input image
    var newImg = image.createImg(image.width, image.height);
    // the filter window will be [-winR, winR];
    var winR = Math.round(sigma*2);
    // ----------- STUDENT CODE BEGIN ------------
    // ----------- STUDENT CODE END ------------
    Gui.alertOnce ('gaussianFilter is not implemented yet');
    return newImg;
};

Filters.edgeFilter = function( image ) {
    // ----------- STUDENT CODE BEGIN ------------
    // ----------- STUDENT CODE END ------------
    Gui.alertOnce ('edgeFilter is not implemented yet');
    return image;
};

Filters.sharpenFilter = function( image ) {
    // ----------- STUDENT CODE BEGIN ------------
    // ----------- STUDENT CODE END ------------
    Gui.alertOnce ('sharpenFilter is not implemented yet');
    return image;
};

Filters.medianFilter = function( image, winR ) {
    // winR: the window will be  [-winR, winR];
    // ----------- STUDENT CODE BEGIN ------------
    // ----------- STUDENT CODE END ------------
    Gui.alertOnce ('medianFilter is not implemented yet');
    return image;
};

Filters.bilateralFilter = function( image, value ) {
    // value could be color [colorSigma, positionSigma];
    // in Gui, we only control color's sigma
    var sigmaR, sigmaS;
    if (value.length > 1) {
        sigmaR = value[0];
        sigmaS = value[1];
    } else {
        sigmaR = value;
        sigmaS = 1;        
    }
    
    // ----------- STUDENT CODE BEGIN ------------
    // ----------- STUDENT CODE END ------------
    Gui.alertOnce ('bilateralFilter is not implemented yet');
    return image;
};

Filters.quantizeFilter = function( image, numBits ) {
    // ----------- STUDENT CODE BEGIN ------------
    // ----------- STUDENT CODE END ------------
    Gui.alertOnce ('quantizeFilter is not implemented yet');
    return image;

};

Filters.randomFilter = function( image, numBits ) {
    // ----------- STUDENT CODE BEGIN ------------
    // ----------- STUDENT CODE END ------------
    Gui.alertOnce ('randomFilter is not implemented yet');
    return image;
            
};

Filters.orderedFilter = function( image, numBits ) {
    // ----------- STUDENT CODE BEGIN ------------
    // ----------- STUDENT CODE END ------------
    Gui.alertOnce ('orderedFilter is not implemented yet');
    return image;
            
};

Filters.floydFilter = function( image, numBits ) {
    // ----------- STUDENT CODE BEGIN ------------
    // ----------- STUDENT CODE END ------------
    Gui.alertOnce ('floydFilter is not implemented yet');
    return image;
};

Filters.scaleFilter = function( image, ratio ) {
    // ----------- STUDENT CODE BEGIN ------------
    // ----------- STUDENT CODE END ------------
    Gui.alertOnce ('scaleFilter is not implemented yet');
    return image;
};

Filters.rotateFilter = function( image, value ) {
    // Note: set pixels outside the image to RGBA(0,0,0,0)
    var angle = value * Math.PI;
    // ----------- STUDENT CODE BEGIN ------------
    // ----------- STUDENT CODE END ------------
    Gui.alertOnce ('rotateFilter is not implemented yet');
    return image;
};

Filters.swirlFilter = function( image, value ) {
    var angle = value * 2.0 * Math.PI;
    // ----------- STUDENT CODE BEGIN ------------
    // ----------- STUDENT CODE END ------------
    Gui.alertOnce ('swirlFilter is not implemented yet');
    return image;
};

Filters.compositeFilterFile = function( backgroundImg, foregroundImg, alphaImg ) {
    // ----------- STUDENT CODE BEGIN ------------
    // ----------- STUDENT CODE END ------------
    Gui.alertOnce ('compositeFilter is not implemented yet');
    return image;
};
Filters.compositeFilter = function( backgroundImg, foregroundImg, alpha ) {
    // ----------- STUDENT CODE BEGIN ------------
    // ----------- STUDENT CODE END ------------
    Gui.alertOnce ('compositeFilter is not implemented yet');
    return image;
};

Filters.morphFilter = function( initialImg, finalImg, lines, alpha ) {
     
    // ----------- STUDENT CODE BEGIN ------------
    // ----------- STUDENT CODE END ------------
    Gui.alertOnce ('morphFilter is not implemented yet');
    return image;
};


Filters.paintFilter = function( image, value ) {
    // ----------- STUDENT CODE BEGIN ------------
    // ----------- STUDENT CODE END ------------
    Gui.alertOnce ('compositeFilter is not implemented yet');
    return image;
};

Filters.xDoGFilter = function( image, value ) {
    // value could be an array
    // ----------- STUDENT CODE BEGIN ------------
    // ----------- STUDENT CODE END ------------
    Gui.alertOnce ('xDoGFilter is not implemented yet');
    return image;
};

Filters.paletteFilter = function( image, colorNum ) {
    // ----------- STUDENT CODE BEGIN ------------
    // ----------- STUDENT CODE END ------------
    Gui.alertOnce ('paletteFilter is not implemented yet');
    return image;
};

Filters.artFilter = function( image, value ) {
    // You can use this filter to do whatever you want, for example
    // trying out some new idea or implementing something for the
    // art contest.
    // Currently the 'value' argument will be 1 or whatever else you set
    // it to in the URL. You could use this value to switch between
    // a bunch of different versions of your code if you want to 
    // code up a bunch of different things for the art contest.
    // ----------- STUDENT CODE BEGIN ------------
    // ----------- STUDENT CODE END ------------
    Gui.alertOnce ('art is not implemented yet');
    return image;
};

// gui params have changed; run the image through all the active filters
Filters.update = function( image, values ) {
    var time = performance.now();
    
    this.sampling = values['sample'];
    
    for ( var prop in values ) {
        if ( (prop in Gui.defaults) && (values[prop] !== Gui.defaults[prop]) ) {
            var val = values[prop];
            if (Gui.propIsStringMenuItem(prop)) {
                val = parseInt(val.trim());
            }
            image = Filters[prop+'Filter']( image, val );
        }
    }
    time = Math.round( performance.now() - time );
    console.log('Filter operation took ' + time + ' ms.');
    
    return image;
};